from __future__ import absolute_import, division, print_function, unicode_literals

import logging

from horizont.lda import LDA


logging.basicConfig(level=logging.INFO)
logger = logging.getLogger('horizont')
logger.addHandler(logging.NullHandler())

# following PEP 386
__version__ = "0.0.5"



from __future__ import absolute_import, division, print_function, unicode_literals

import io
import os
import unittest

import numpy as np

import horizont.utils as utils
import horizont._utils as _utils


class TestUtils(unittest.TestCase):

    np.random.seed(99)

    D = 100
    W = 50
    N_WORDS_PER_DOC = 500
    N = W * N_WORDS_PER_DOC
    dtm = np.zeros((D, W), dtype=int)
    for d in range(D):
        dtm[d] = np.random.multinomial(N_WORDS_PER_DOC, np.ones(W) * 1/W)
    N_BY_W = np.sum(dtm, axis=0)
    N_BY_D = np.sum(dtm, axis=1)

    def test_setup(self):
        dtm, D, N_WORDS_PER_DOC = self.dtm, self.D, self.N_WORDS_PER_DOC
        self.assertEqual(np.sum(dtm), D * N_WORDS_PER_DOC)

    def test_matrix_to_lists(self):
        dtm, D, N_WORDS_PER_DOC = self.dtm, self.D, self.N_WORDS_PER_DOC
        N_BY_D, N_BY_W = self.N_BY_D, self.N_BY_W
        WS, DS = utils.matrix_to_lists(dtm)
        self.assertEqual(len(WS), D * N_WORDS_PER_DOC)
        self.assertEqual(len(WS), len(DS))
        self.assertEqual(dtm.shape, (max(DS) + 1, max(WS) + 1))
        self.assertTrue(all(DS == sorted(DS)))
        self.assertTrue(np.all(np.bincount(DS) == N_BY_D))
        self.assertTrue(np.all(np.bincount(WS) == N_BY_W))

    def test_lists_to_matrix(self):
        dtm = self.dtm
        WS, DS = utils.matrix_to_lists(dtm)
        dtm_new = utils.lists_to_matrix(WS, DS)
        self.assertTrue(np.all(dtm == dtm_new))

    def test_ldac2dtm_offset(self):
        test_dir = os.path.dirname(__file__)
        ap_ldac_fn = os.path.join(test_dir, 'ap.dat')
        with self.assertRaises(ValueError):
            utils.ldac2dtm(open(ap_ldac_fn), offset=1)

    def test_ldac2dtm(self):
        test_dir = os.path.dirname(__file__)
        ap_ldac_fn = os.path.join(test_dir, 'ch.ldac')
        dtm = utils.ldac2dtm(open(ap_ldac_fn))
        self.assertEqual(dtm.shape, (395, 4258))
        self.assertEqual(dtm.sum(), 84010)

    def test_ldac_conversion(self):
        dtm = self.dtm
        N, V = dtm.shape
        doclines = list(utils.dtm2ldac(self.dtm))
        nd_unique = np.sum(dtm > 0, axis=1)
        for n, docline in zip(nd_unique, doclines):
            self.assertEqual(n, int(docline.split(' ')[0]))
        self.assertEqual(len(doclines), N)
        f = io.StringIO('\n'.join(doclines))
        dtm_new = utils.ldac2dtm(f)
        self.assertTrue(np.all(dtm == dtm_new))

    def test_searchsorted(self):
        for _ in range(10000):
            arr = np.sort(np.random.rand(10))
            r = np.random.rand()
            self.assertEqual(np.searchsorted(arr, r), _utils.searchsorted(arr, r))
        r = -1
        self.assertEqual(np.searchsorted(arr, r), _utils.searchsorted(arr, r))
        r = 10
        self.assertEqual(np.searchsorted(arr, r), _utils.searchsorted(arr, r))

    def test_choice(self):
        r = 0.9851367976958045
        probz = np.array([0.14800396, 0.29600989, 0.46001528])
        z = _utils.choice(probz, r)
        self.assertEqual(z, len(probz) - 1)

        N = 10000
        K = 100
        samples = []
        random_state = np.random.RandomState(1)
        for _ in range(N):
            # unnormalized probability distribution
            dist = random_state.randint(10, 1000, size=K).astype(float)
            samples.append(_utils.choice(dist, random_state.rand()))
        self.assertGreaterEqual(max(samples), 0)
        self.assertLessEqual(max(samples), K-1)

        p = 1/K
        bound = 3*np.sqrt(N*p*(1-p))
        counts = np.bincount(samples)
        for k in range(K):
            self.assertLess(abs(counts[k] - N*p), bound)

    def test_choice_consistency(self):
        N = 1000
        K = 100
        samples = []
        random_state = np.random.RandomState(1)
        for _ in range(N):
            # unnormalized probability distribution
            dist = random_state.randint(10, 1000, size=K).astype(float)
            samples.append(_utils.choice(dist, random_state.rand()))

        samples2 = []
        random_state = np.random.RandomState(1)
        for _ in range(N):
            # unnormalized probability distribution
            dist = random_state.randint(10, 1000, size=K).astype(float)
            samples2.append(_utils.choice(dist, random_state.rand()))
        self.assertEqual(samples, samples2)



#cython: language_level=3
#cython: boundscheck=False
#cython: wraparound=False

cimport cython
cimport cpython.array
cimport libc.math as math
from libc.stdlib cimport malloc, free
cimport numpy as np

import sys
import numpy as np

import horizont.utils
import horizont._utils


@cython.cdivision(True)
def _sample_topics(int[:] WS, int[:] DS, int[:] ZS, int[:, :] nzw, int[:, :] ndz, int[:] nz,
                   double[:] alpha, double[:] eta, double[:] rands):
    cdef int i, k, w, d, z, z_new, n_topics
    cdef int n_rand = len(rands)
    cdef double r, dist_cum
    cdef double alpha_sum = sum(alpha)
    cdef double eta_sum = sum(eta)
    n_topics = len(nzw)

    cdef double * dist_sum_ptr = <double*> malloc(sizeof(double) * n_topics)
    cdef double[:] dist_sum = <double [:n_topics]> dist_sum_ptr

    for i in range(len(WS)):
        w = WS[i]
        d = DS[i]
        z = ZS[i]

        nzw[z, w] -= 1
        ndz[d, z] -= 1
        nz[z] -= 1

        dist_cum = 0
        for k in range(n_topics):
            # eta is a double so cdivision yields a double
            dist_cum += (nzw[k, w] + eta[w]) / (nz[k] + eta_sum) * (ndz[d, k] + alpha[k])
            dist_sum[k] = dist_cum

        r = rands[i % n_rand] * dist_cum  # dist_cum == dist_sum[-1]
        # XXX: using cimport and a pxd file for searchsorted might be faster
        z_new = horizont._utils.searchsorted(dist_sum, r)

        ZS[i] = z_new
        nzw[z_new, w] += 1
        ndz[d, z_new] += 1
        nz[z_new] += 1

    free(<void *> dist_sum_ptr)


@cython.cdivision(True)
cdef _sample_z(int w, np.ndarray[np.int_t] nz, np.ndarray[np.float_t, ndim=2] Phi, double alpha, r):
    """
    Sample new z given w, nz, and Phi.
    r is a uniform random variate.
    Assumes symmetric Dirichlet.
    """
    n_topics = len(nz)
    probz = np.empty(n_topics)
    for k in range(n_topics):
        probz[k] = Phi[k, w] * nz[k] + alpha
    z_new = horizont._utils.choice(probz, r)
    return z_new


@cython.cdivision(True)
def _score_doc(np.ndarray[np.int_t] x, np.ndarray[np.float_t, ndim=2] Phi,
               double alpha, int R, np.ndarray[np.float_t] rands):
    WS, _ = horizont.utils.matrix_to_lists(np.atleast_2d(x))
    ZS = np.zeros_like(WS)

    n_rand = len(rands)
    i = 0  # index for random draws

    K = len(Phi)
    ll = 0
    for n, w in enumerate(WS):
        pn = 0
        # XXX: tracking just the histogram of topic counts might be faster
        for r in range(R):
            for nprime in range(n):
                r = rands[i % n_rand]
                i += 1
                nz = np.bincount(np.delete(ZS[:n], nprime), minlength=K)
                ZS[nprime] = _sample_z(WS[nprime], nz, Phi, alpha, r)
            nz = np.bincount(ZS[:n], minlength=K)
            Etheta = nz + alpha
            Etheta = 1.0 * Etheta / np.sum(Etheta)
            pn += np.dot(Phi[:, w], Etheta)
        pn = 1.0 * pn / R
        ll += np.log(pn)
    return ll



import unittest

import numpy as np

import horizont
import horizont.random
import horizont._random
import horizont.utils


class TestPolyaGamma(unittest.TestCase):
    """Test the different PG random variable generation routines"""

    def setUp(self):
        np.random.seed(5)

    def test_rng(self):
        b, c = 200, 0
        r1 = horizont.random.pg(b, c)
        r2 = horizont.random.pg(b, c)
        self.assertNotEqual(r1, r2)

    def test_moments(self):
        # test that moments are positive
        b, c = 1400, -3.7
        m1 = horizont._random.pg_m1(b, c)
        self.assertGreater(m1, 1)
        m2 = horizont._random.pg_m2(b, c)
        self.assertGreater(m2, 1)

        b, c = 1400, -2.62
        m1 = horizont._random.pg_m1(b, c)
        self.assertGreater(m1, 1)
        m2 = horizont._random.pg_m2(b, c)
        self.assertGreater(m2, 1)

    def test_neg_param(self):
        b, c = 1400, -3.78
        r = horizont.random.pg(b, c)
        self.assertGreater(r, 0)

        b, c = 1400, -2.62
        m1 = horizont._random.pg_m1(b, c)
        self.assertGreater(m1, 1)
        m2 = horizont._random.pg_m2(b, c)
        self.assertGreater(m2, 1)
        self.assertGreater(m2 - m1*m1, 1)
        r = horizont.random.pg(b, c)
        self.assertGreater(r, 0)

    def test_rng_with_seed(self):
        b, c = 200, 0
        seed = 5
        r1 = horizont.random.pg(b, c, seed=seed)
        r2 = horizont.random.pg(b, c, seed=seed)
        self.assertEqual(r1, r2)

        np.random.seed(seed)
        r1 = horizont.random.pg(b, c)
        np.random.seed(seed)
        r2 = horizont.random.pg(b, c)
        self.assertEqual(r1, r2)

    def test_polya_gamma_norm(self):
        # with a high b, a normal approximation is used
        b, c = 200, 0
        S = int(1e4)
        rvs = [horizont.random.pg(b, c) for _ in range(S)]
        m = horizont._random.pg_m1(b, c)
        v = horizont._random.pg_m2(b, c) - m*m
        np.testing.assert_allclose(np.mean(rvs), m, rtol=0.03)
        np.testing.assert_allclose(np.var(rvs, ddof=1), v, rtol=0.03)

        b, c = 200, 3.5
        rvs = [horizont.random.pg(b, c) for _ in range(S)]
        m = horizont._random.pg_m1(b, c)
        v = horizont._random.pg_m2(b, c) - m*m
        np.testing.assert_allclose(np.mean(rvs), m, rtol=0.02)
        np.testing.assert_allclose(np.var(rvs, ddof=1), v, rtol=0.02)

    def test_polya_gamma(self):
        b, c = 2, 0
        S = int(1e4)
        rvs = [horizont.random.pg(b, c) for _ in range(S)]
        m = horizont._random.pg_m1(b, c)
        v = horizont._random.pg_m2(b, c) - m*m
        np.testing.assert_allclose(np.mean(rvs), m, rtol=0.05)
        np.testing.assert_allclose(np.var(rvs, ddof=1), v, rtol=0.05)

        b, c = 2, 3.5
        rvs = [horizont.random.pg(b, c) for _ in range(S)]
        m = horizont._random.pg_m1(b, c)
        v = horizont._random.pg_m2(b, c) - m*m
        np.testing.assert_allclose(np.mean(rvs), m, rtol=0.05)
        np.testing.assert_allclose(np.var(rvs, ddof=1), v, rtol=0.05)

    def test_polya_gamma_alt(self):
        b, c = 9, 0
        S = int(1e4)
        rvs = [horizont.random.pg(b, c) for _ in range(S)]
        m = horizont._random.pg_m1(b, c)
        v = horizont._random.pg_m2(b, c) - m*m
        np.testing.assert_allclose(np.mean(rvs), m, rtol=0.02)
        np.testing.assert_allclose(np.var(rvs, ddof=1), v, rtol=0.02)

        b, c = 9, 3.5
        rvs = [horizont.random.pg(b, c) for _ in range(S)]
        m = horizont._random.pg_m1(b, c)
        v = horizont._random.pg_m2(b, c) - m*m
        np.testing.assert_allclose(np.mean(rvs), m, rtol=0.02)
        np.testing.assert_allclose(np.var(rvs, ddof=1), v, rtol=0.02)

    def test_polya_gamma_sp(self):
        b, c = 20, 0
        S = int(1e4)
        rvs = [horizont.random.pg(b, c) for _ in range(S)]
        m = horizont._random.pg_m1(b, c)
        v = horizont._random.pg_m2(b, c) - m*m
        np.testing.assert_allclose(np.mean(rvs), m, rtol=0.02)
        np.testing.assert_allclose(np.var(rvs, ddof=1), v, rtol=0.02)

        b, c = 20, 3.5
        rvs = [horizont.random.pg(b, c) for _ in range(S)]
        m = horizont._random.pg_m1(b, c)
        v = horizont._random.pg_m2(b, c) - m*m
        np.testing.assert_allclose(np.mean(rvs), m, rtol=0.02)
        np.testing.assert_allclose(np.var(rvs, ddof=1), v, rtol=0.02)



import unittest

import numpy as np
import scipy.misc
import scipy.special
from scipy.stats.mstats import mquantiles

import horizont
import horizont.random
import horizont.utils

trigamma = lambda x: scipy.special.polygamma(1, x)


def softmax(beta):
    return np.exp(beta - scipy.misc.logsumexp(beta))


class TestLogisticNormal(unittest.TestCase):

    def test_logistic_normal_multinomial(self):
        """Verify posterior resembles a posterior with a Dirichlet prior"""

        random_state = np.random.RandomState(5)
        nz = np.array([400, 700, 100, 200])
        J = len(nz)
        N = sum(nz)
        n_iter = 1000
        alpha = 1
        m0 = np.repeat(0, J)
        P0 = np.diag(np.repeat(1/trigamma(alpha), J))

        # summary statistics
        kappa = nz - np.sum(nz)/2

        # initialize
        beta = random_state.multivariate_normal(m0, np.linalg.inv(P0))
        beta[-1] = 0
        omega = np.zeros(J)
        c = np.zeros(J)

        # variables to hold results
        THETA = np.empty((n_iter, J))

        # sampling loop
        for it in range(n_iter):

            for j in range(J):
                c_j = scipy.misc.logsumexp(np.delete(beta, j))
                c[j] = c_j
                eta_j = beta[j] - c_j
                # sample omega[j]
                omega[j] = horizont.random.pg(N, eta_j)
                self.assertGreater(omega[j], 0)

            # sample beta
            P1 = np.diag(omega) + P0
            V1 = np.linalg.inv(P1)
            m1 = np.dot(V1, (kappa + omega*c + np.dot(P0, m0)))
            # this may generate an annoying deprecation warning
            beta = random_state.multivariate_normal(m1, V1)
            beta[-1] = 0

            # save scan
            THETA[it] = softmax(beta)

        burnin = 100
        theta_means = np.mean(THETA[burnin:, :], axis=0)
        dirichlet_post = random_state.dirichlet(alpha + nz, size=2000)
        dirichlet_quantiles = mquantiles(dirichlet_post, prob=[0.25, 0.75], axis=0)
        for i, quantiles in enumerate(dirichlet_quantiles.T):
            self.assertTrue(quantiles[0] < theta_means[i] < quantiles[1])

    def test_logistic_normal_multinomial_wishart(self):
        """Verify posterior resembles a posterior with a Dirichlet prior"""

        random_state = np.random.RandomState(5)
        nz = np.array([400, 700, 100, 200])
        J = len(nz)
        N = sum(nz)
        n_iter = 1000
        alpha = 1
        m0 = np.repeat(0, J)
        P0 = np.diag(np.repeat(1/trigamma(alpha), J))

        # summary statistics
        kappa = nz - np.sum(nz)/2

        # initialize
        beta = random_state.multivariate_normal(m0, np.linalg.inv(P0))
        beta[-1] = 0
        omega = np.zeros(J)
        c = np.zeros(J)

        # variables to hold results
        THETA = np.empty((n_iter, J))
        SIGMA = np.empty((n_iter, J, J))

        # sampling loop
        for it in range(n_iter):

            for j in range(J):
                c_j = scipy.misc.logsumexp(np.delete(beta, j))
                c[j] = c_j
                eta_j = beta[j] - c_j
                # sample omega[j]
                omega[j] = horizont.random.pg(N, eta_j)
                self.assertGreater(omega[j], 0)

            # sample beta
            P1 = np.diag(omega) + P0
            V1 = np.linalg.inv(P1)
            m1 = np.dot(V1, (kappa + omega*c + np.dot(P0, m0)))
            # this may generate an annoying deprecation warning
            beta = random_state.multivariate_normal(m1, V1)
            beta[-1] = 0

            # save scan
            THETA[it] = softmax(beta)

        burnin = 100
        theta_means = np.mean(THETA[burnin:, :], axis=0)
        dirichlet_post = random_state.dirichlet(alpha + nz, size=2000)
        dirichlet_quantiles = mquantiles(dirichlet_post, prob=[0.25, 0.75], axis=0)
        for i, quantiles in enumerate(dirichlet_quantiles.T):
            self.assertTrue(quantiles[0] < theta_means[i] < quantiles[1])



# coding=utf-8
"""
Latent Dirichlet Allocation with Gibbs sampling
"""
from __future__ import absolute_import, division, print_function, unicode_literals
try:
    import concurrent.futures as futures
except ImportError:
    import futures
import logging
import os
import sys

import numpy as np
import scipy.special
import sklearn.base
import sklearn.utils

import horizont._lda
import horizont.utils

PY2 = sys.version_info[0] == 2
if PY2:
    from itertools import izip as zip


logger = logging.getLogger('horizont')


class LDA(sklearn.base.BaseEstimator, sklearn.base.TransformerMixin):
    """Latent Dirichlet allocation using Gibbs sampling

    Parameters
    ----------
    n_topics : int
        Number of topics

    n_iter : int, default 1000
        Number of sampling iterations

    alpha : float, default 0.1
        Dirichlet parameter for distribution over topics

    eta : float, default 0.01
        Dirichlet parameter for distribution over words

    random_state : numpy.RandomState | int, optional
        The generator used for the initial topics. Default: numpy.random

    Attributes
    ----------
    `components_` : array, shape = [n_topics, n_features]
        Point estimate of the topic-word distributions (denoted Phi in literature)
    `phi_` : array, shape = [n_topics, n_features]
        Alias for components_
    `nzw_` : array, shape = [n_topics, n_features]
        Matrix of counts recording topic-word assignments in final iteration.
    `theta_` : array, shape = [n_samples, n_features]
        Point estimate of the document-topic distributions (denoted Theta in literature)
    `ndz_` : array, shape = [n_samples, n_features]
        Matrix of counts recording document-topic assignments in final iteration.
    `nz_` : array, shape = [n_topics]
        Array of topic assignment counts in final iteration.

    Examples
    --------
    >>> import numpy as np
    >>> X = np.array([[1,1], [2, 1], [3, 1], [4, 1], [5, 8], [6, 1]])
    >>> from horizont import LDA
    >>> model = LDA(n_topics=2, random_state=0, n_iter=100)
    >>> model.fit(X) #doctest: +ELLIPSIS +NORMALIZE_WHITESPACE
    LDA(alpha=...
    >>> model.components_
    array([[ 0.85714286,  0.14285714],
           [ 0.45      ,  0.55      ]])
    >>> model.loglikelihood() #doctest: +ELLIPSIS
    -40.395...

    Note
    ----
    Although this module uses code written in C (generated by Cython) in its
    implementation of LDA, sampling is *considerably* slower than in pure-Java
    (MALLET_) or pure C implementations (`HCA
    <http://www.mloss.org/software/view/527/>`_). Consider using different
    software if you are working a large dataset.

    References
    ----------
    Blei, David M., Andrew Y. Ng, and Michael I. Jordan. "Latent Dirichlet
    Allocation." Journal of Machine Learning Research 3 (2003): 993–1022.

    Griffiths, Thomas L., and Mark Steyvers. "Finding Scientific Topics."
    Proceedings of the National Academy of Sciences 101 (2004): 5228–5235.
    doi:10.1073/pnas.0307752101.

    Wallach, Hanna, David Mimno, and Andrew McCallum. "Rethinking LDA: Why
    Priors Matter." In Advances in Neural Information Processing Systems 22,
    edited by Y.  Bengio, D. Schuurmans, J. Lafferty, C. K. I. Williams, and A.
    Culotta, 1973–1981, 2009.
    """

    def __init__(self, n_topics=None, n_iter=1000, alpha=0.1, eta=0.01, random_state=None):
        self.n_topics = n_topics
        self.n_iter = n_iter
        self.alpha = alpha
        self.eta = eta
        self.random_state = random_state
        rng = sklearn.utils.check_random_state(random_state)
        # random numbers that are reused
        self._rands = rng.rand(1000)

    def fit(self, X, y=None):
        """Fit the model with X.

        Parameters
        ----------
        X: array-like, shape (n_samples, n_features)
            Training data, where n_samples in the number of samples
            and n_features is the number of features.

        Returns
        -------
        self : object
            Returns the instance itself.
        """
        self._fit(X)
        return self

    def _fit(self, X):
        """Fit the model to the data X.

        Parameters
        ----------
        X: array-like, shape (n_samples, n_features)
            Training vector, where n_samples in the number of samples and
            n_features is the number of features.
        """
        random_state = sklearn.utils.check_random_state(self.random_state)
        X = np.atleast_2d(sklearn.utils.as_float_array(X))
        self._initialize(X, random_state)
        for it in range(self.n_iter):
            if it % 10 == 0:
                self._print_status(it)
            else:
                logger.info("<{}>".format(it))
            self._sample_topics(random_state)
        self._print_status(self.n_iter)
        self.components_ = self.nzw_ + self.eta
        self.components_ /= np.sum(self.components_, axis=1, keepdims=True)
        self.phi_ = self.components_
        self.theta_ = self.ndz_ + self.alpha
        self.theta_ /= np.sum(self.theta_, axis=1, keepdims=True)

        # delete attributes no longer needed after fitting
        del self.WS
        del self.DS
        del self.ZS
        return self

    def _print_status(self, iter):
        ll = self.loglikelihood()
        N = len(self.WS)
        logger.info("<{}> log likelihood: {:.0f}, log perp: {:.4f}".format(iter, ll, -1 * ll / N))

    def _initialize(self, X, random_state):
        D, W = X.shape
        N = int(np.sum(X))
        n_topics = self.n_topics
        n_iter = self.n_iter
        logger.info("n_documents: {}".format(D))
        logger.info("vocab_size: {}".format(W))
        logger.info("n_words: {}".format(N))
        logger.info("n_topics: {}".format(n_topics))
        logger.info("n_iter: {}".format(n_iter))

        self.nzw_ = np.zeros((n_topics, W), dtype=np.intc)
        self.ndz_ = np.zeros((D, n_topics), dtype=np.intc)
        self.nz_ = np.zeros(n_topics, dtype=np.intc)

        # could be moved into Cython
        self.WS, self.DS = horizont.utils.matrix_to_lists(X)
        self.ZS = np.zeros_like(self.WS, dtype=np.intc)
        for i, (w, d) in enumerate(zip(self.WS, self.DS)):
            # random initialization
            # FIXME: improve initialization
            # FIXME: initialization could occur elsewhere
            z_new = random_state.randint(n_topics)
            self.ZS[i] = z_new
            self.ndz_[d, z_new] += 1
            self.nzw_[z_new, w] += 1
            self.nz_[z_new] += 1

    def loglikelihood(self):
        nzw, ndz, nz = self.nzw_, self.ndz_, self.nz_
        alpha = self.alpha
        eta = self.eta
        return self._loglikelihood(nzw, ndz, nz, alpha, eta)

    @staticmethod
    def _loglikelihood(nzw, ndz, nz, alpha, eta):
        """
        Calculate the complete log likelihood, log p(w,z).

        log p(w,z) = log p(w|z) + log p(z)
        """
        D, n_topics = ndz.shape
        vocab_size = nzw.shape[1]
        nd = np.sum(ndz, axis=1)

        ll = 0.0

        # calculate log p(w|z)
        gammaln_eta = scipy.special.gammaln(eta)
        gammaln_alpha = scipy.special.gammaln(alpha)

        ll += n_topics * scipy.special.gammaln(eta * vocab_size)
        for k in range(n_topics):
            ll -= scipy.special.gammaln(eta * vocab_size + nz[k])
            for w in range(vocab_size):
                # if nzw[k, w] == 0 addition and subtraction cancel out
                if nzw[k, w] > 0:
                    ll += scipy.special.gammaln(eta + nzw[k, w]) - gammaln_eta

        # calculate log p(z)
        for d in range(D):
            ll += scipy.special.gammaln(alpha * n_topics) - scipy.special.gammaln(alpha * n_topics + nd[d])
            for k in range(n_topics):
                if ndz[d, k] > 0:
                    ll += scipy.special.gammaln(alpha + ndz[d, k]) - gammaln_alpha
        return ll

    def _sample_topics(self, random_state):
        random_state = sklearn.utils.check_random_state(self.random_state)
        rands = self._rands
        random_state.shuffle(rands)
        n_topics, vocab_size = self.nzw_.shape
        alpha = np.repeat(self.alpha, n_topics).astype(float)
        eta = np.repeat(self.eta, vocab_size).astype(float)
        horizont._lda._sample_topics(self.WS, self.DS, self.ZS, self.nzw_, self.ndz_, self.nz_, alpha, eta, rands)

    def transform(self, X, y=None):
        """Transform the data X according to the fitted model

        Parameters
        ----------
        X : array-like, shape (n_samples, n_features)
            New data, where n_samples in the number of samples
            and n_features is the number of features.

        Returns
        -------
        theta : array-like, shape (n_samples, n_topics)
            Point estimate of the document-topic distributions

        """
        raise NotImplementedError

    def fit_transform(self, X, y=None):
        """Apply dimensionality reduction on X.

        Parameters
        ----------
        X : array-like, shape (n_samples, n_features)
            New data, where n_samples in the number of samples
            and n_features is the number of features.

        Returns
        -------
        theta : array-like, shape (n_samples, n_topics)
            Point estimate of the document-topic distributions

        """
        self._fit(np.atleast_2d(X))
        return self.theta_

    def score(self, X, R, random_state=None):
        """
        Calculate marginal probability of observations in X given Phi.

        Returns a list with estimates for each document separately, mimicking
        the behavior of scikit-learn. Uses Buntine's left-to-right sequential sampler.

        Parameters
        ----------
        X : array, [n_samples, n_features]
            The document-term matrix of documents for evaluation.

        R : int
            The number of particles to use for the estimation.

        Returns
        -------
        logprobs : array of length n_samples
            Estimate of marginal log probability for each row of X.
        """
        if random_state is None:
            random_state = sklearn.utils.check_random_state(self.random_state)
            rands = self._rands
        else:
            random_state = sklearn.utils.check_random_state(random_state)
            # get rands in a known state
            rands = np.sort(self._rands)
            random_state.shuffle(rands)

        N, V = X.shape
        Phi, alpha = self.phi_, self.alpha
        np.testing.assert_equal(V, Phi.shape[1])

        # calculate marginal probability for each document separately
        logprobs = []
        multiprocessing = int(os.environ.get('JOBLIB_MULTIPROCESSING', 1)) or None
        if multiprocessing:
            with futures.ProcessPoolExecutor() as ex:
                futs = []
                for x in X:
                    # for consistency one needs to pass a copy of rands
                    # the executor takes time to pickle; it's not instantaneous
                    futs.append(ex.submit(horizont._lda._score_doc, x, Phi, alpha, R, rands.copy()))
                    random_state.shuffle(rands)
                for i, fut in enumerate(futs):
                    logprobs.append(fut.result())
        else:
            for x in X:
                logprobs.append(horizont._lda._score_doc(x, Phi, alpha, R, rands))
                random_state.shuffle(rands)
        return logprobs



#cython: language_level=3
#cython: boundscheck=False
#cython: wraparound=False

from libc.stdlib cimport malloc, free

cpdef int searchsorted(double[:] arr, double v):
    """
    Find index where an element should be inserted to maintain order.
    
    This is a Cython version of numpy.searchsorted (bisection search).
    """
    cdef int imin, imax, imid
    imin = 0
    imax = len(arr)
    while imin < imax:
        imid = imin + ((imax - imin) >> 2)
        if v > arr[imid]:
            imin = imid + 1
        else:
            imax = imid
    return imin


cpdef int choice(double[:] p, double r):
    """
    Cython version of numpy.random.choice where uniform random variate r must
    be provided.
    """
    cdef int k, z
    cdef int K = len(p)
    cdef double dist_cum = 0
    cdef double * dist_sum_ptr = <double*> malloc(sizeof(double) * K)
    cdef double[:] dist_sum = <double [:K]> dist_sum_ptr
    for k in range(K):
        dist_cum += p[k]
        dist_sum[k] = dist_cum
    z = searchsorted(dist_sum, dist_cum * r)
    free(<void *> dist_sum_ptr)
    return z



from __future__ import absolute_import, division, print_function, unicode_literals

import unittest

import numpy as np

np.random.seed(1)

from horizont.metrics import kl_div, js_div


class TestMetrics(unittest.TestCase):

    def test_kl_div(self):
        X = [[0.7, 0.3], [0.5, 0.5]]
        dist = kl_div(X)
        desired = np.array([[0., 0.082282878505051782],
                            [0.087176693572388914, 0.]])
        np.testing.assert_allclose(dist, desired)

    def test_js_div(self):
        X = [[0.7, 0.3], [0.5, 0.5]]
        dist = js_div(X)
        desired = np.array([[0., 0.082282878505051782],
                            [0.087176693572388914, 0.]])
        desired = 0.5 * (desired + desired.T)
        np.testing.assert_allclose(dist, desired)



# distutils: language = c++
#cython: language_level=3
#cython: boundscheck=False
#cython: wraparound=False

from cython.operator cimport dereference as deref
from libc.math cimport sqrt

import numpy as np

cdef extern from "RNG/GRNG.hpp":
    cdef cppclass BasicRNG:
        BasicRNG()
        double norm(double mean, double sd)
        void set(unsigned long seed)

cdef extern from "RNG/RNG.hpp":
    cdef cppclass RNG(BasicRNG):
        RNG()

cdef extern from "BayesLogit/Code/C/PolyaGamma.h":
    cdef cppclass PolyaGamma:
        double pg_m1(double b, double z)
        double pg_m2(double b, double z)
        double draw(int n, double z, RNG& r)
        double draw_sum_of_gammas(double n, double z, RNG& r)

cdef extern from "BayesLogit/Code/C/PolyaGammaAlt.h":
    cdef cppclass PolyaGammaAlt:
        double draw(double h, double z, RNG& r)

cdef extern from "BayesLogit/Code/C/PolyaGammaSP.h":
    cdef cppclass PolyaGammaSP:
        int draw(double& d, double h, double z, RNG& r)

# REF: BayesLogit/Code/C/LogitWrapper.cpp
# XXX: there is now a parallel implementation see ParallelWrapper.cpp
cdef rpg_hybrid(double *x, double *h, double *z, int* num, unsigned long seed):
    cdef:
        RNG r = RNG()
        PolyaGamma dv
        PolyaGammaAlt alt
        PolyaGammaSP sp
        int i
        double b, c, m, v

    # works more readily than inheriting constructor from BasicRNG
    r.set(seed)

    # Cython optimizes this to a pure C loop
    for i in range(deref(num)):
        b = h[i]
        c = z[i]
        if b > 170:
            m = dv.pg_m1(b, c)
            v = dv.pg_m2(b, c) - m*m
            x[i] = r.norm(m, sqrt(v))
        elif b > 13:
            sp.draw(x[i], b, c, r)
        elif  b == 1 or b == 2:
            x[i] = dv.draw(<int>b, c, r)
        elif b > 1:
            x[i] = alt.draw(b, c, r);
        elif b > 0:
            x[i] = dv.draw_sum_of_gammas(b, c, r)
        else:
            x[i] = 0.0


def pg(double b, double z, unsigned long seed):
    """Draw once from a Pólya-Gamma distribution"""

    # TODO: vectorize this, see https://github.com/cython/cython/wiki/tutorials-NumpyPointerToC
    cdef:
        int size = 1
        double x
    rpg_hybrid(&x, &b, &z, &size, seed)
    return x


def pg_m1(double b, double z):
    """First moment of PG(b, z)"""
    cdef:
        PolyaGamma dv
    return dv.pg_m1(b, z)


def pg_m2(double b, double z):
    """Second moment of PG(b, z)"""
    cdef:
        PolyaGamma dv
    return dv.pg_m2(b, z)



from __future__ import absolute_import, division, print_function, unicode_literals

import os
import tempfile

import numpy as np
import scipy.misc

import horizont
import horizont.metrics
from horizont.tests import base
import horizont.utils

NUM_TOPICS = 10
DOC_LENGTH = 100
NUM_DOCS = 1000

np.random.seed(1)


def _loglikelihood_conditional(X, THETA, PHI):
    """
    Calculate the conditional loglikelihood under LDA of the document-term
    matrix X given \Theta and \Phi

    For example, for a single word w
    p(w|\Theta, \Phi) = \sum_k p(w|z = k, \Phi) p(z = k|\Theta)
    """
    ll = 0
    np.testing.assert_equal(len(X), len(THETA))
    LOGTHETA = np.log(THETA)
    LOGPHI = np.log(PHI)
    for d, doc in enumerate(X):
        for v, cnt in enumerate(doc):
            if cnt != 0:
                ll += cnt * scipy.misc.logsumexp(LOGPHI[:, v] + LOGTHETA[d, :])
    return ll


def make_topic(width, index, fill_col=False):
    """
    Return topic distribution whose "words" form a vertical or horizontal stripe.
    """
    topic = np.zeros((width, width), dtype=float)
    if fill_col:
        topic[:, index] = 1
    else:
        topic[index, :] = 1
    topic = topic.flatten()
    return topic / np.sum(topic)


def make_topics(num_topics):
    """
    Make topic distributions for the corpus.
    """
    width = num_topics // 2
    vocab_size = width ** 2
    doctopic = np.zeros((num_topics, vocab_size))
    for i in range(width):
        doctopic[i] = make_topic(width, i)
    for i in range(width):
        doctopic[width + i] = make_topic(width, i, fill_col=True)
    return doctopic


def print_topic(topic):
    """
    Print a topic to the console
    """
    vocab_size = len(topic)
    width = int(np.sqrt(vocab_size))
    topic = np.round(topic, 3)
    for i in range(width):
        startidx = i * width
        stopidx = (i+1) * width
        row = topic[startidx:stopidx]
        print(' '.join('{:.2f}'.format(v) for v in row))


def print_topics(topics):
    for i, t in enumerate(topics):
        print("TOPIC {}:".format(i))
        print_topic(t)


def make_document(topics, doc_length, alpha=0.1):
    num_topics, vocab_size = topics.shape
    doctopic = np.random.dirichlet([alpha] * num_topics)
    document = np.zeros(vocab_size, dtype=int)
    for _ in range(doc_length):
        z = np.random.choice(num_topics, p=doctopic)
        w = np.random.choice(vocab_size, p=topics[z])
        document[w] += 1
    return document


def make_corpus(topics, num_docs, doc_length, alpha=0.1):
    """
    Returns a document-term matrix of counts.
    """
    num_topics, vocab_size = topics.shape
    dtm = np.zeros((num_docs, vocab_size), dtype=int)
    for i in range(num_docs):
        dtm[i] = make_document(topics, doc_length, alpha)
    return dtm


def identify_topic(unknown_topic, topics):
    """
    Returns row index of closest topic in `topics`
    """
    # pending an improved implementation of kl_div
    X = np.vstack([topics, unknown_topic])
    np.testing.assert_equal(len(X), len(topics) + 1)
    dist = horizont.metrics.kl_div(X+1e-7)  # avoid np.log(0)
    closest_topic_index = np.delete(dist[-1], -1).argmin()
    return closest_topic_index


def order_topics(unordered, reference):
    """
    Sort ``unordered`` by matching it to closest rows in ``reference``.
    """
    np.testing.assert_equal(unordered.shape, reference.shape)
    row_map = []
    for row in unordered:
        row_map.append(identify_topic(row, reference))
    if len(set(row_map)) != len(row_map):
        raise ValueError("Unable to uniquely sort topics")
    return unordered[np.argsort(row_map)]


def sparsify_topics(topics):
    """
    Bluntly sparsify topic distributions.

    Take the top ``width`` values and make everything else zero.
    """
    num_topics = len(topics)
    width = num_topics // 2
    topics = topics.copy()
    for row in topics:
        lowest_indexes = row.argsort()[:-width]
        row[lowest_indexes] = 0
        row /= sum(row)
    return topics


class TestLDAStrips(base.TestCase):

    topics = make_topics(NUM_TOPICS)
    dtm = make_corpus(topics, NUM_DOCS, DOC_LENGTH)
    tempdir = tempfile.mkdtemp()
    with open(os.path.join(tempdir, 'strips.ldac'), 'w') as f:
        f.write('\n'.join(horizont.utils.dtm2ldac(dtm)))
        f.write('\n')

    def test_topics(self):
        topics = self.topics
        width = NUM_TOPICS // 2
        np.testing.assert_equal(topics.shape, (NUM_TOPICS, width**2))
        np.testing.assert_almost_equal(np.sum(topics), NUM_TOPICS)
        self.assertTrue(all(topics.sum(axis=1) == 1))
        self.assertTrue(all(topics.sum(axis=0) == 0.4))

    def test_make_document(self):
        topics = self.topics
        num_topics, vocab_size = topics.shape
        doc = make_document(topics, DOC_LENGTH)
        self.assertEqual(sum(doc), DOC_LENGTH)

        docs = [make_document(topics, DOC_LENGTH) for _ in range(1000)]
        np.testing.assert_allclose(sum(docs) / len(docs), DOC_LENGTH / vocab_size, 0.2)

    def test_make_corpus(self):
        topics = self.topics
        dtm = self.dtm
        num_topics, vocab_size = topics.shape
        self.assertEqual(dtm.shape, (NUM_DOCS, vocab_size))
        np.testing.assert_equal(np.sum(dtm, axis=1), DOC_LENGTH)
        np.testing.assert_allclose(np.sum(dtm, axis=0) / len(dtm), DOC_LENGTH / vocab_size, 0.2)

    def test_identify_topic(self):
        topics = self.topics
        num_topics, vocab_size = topics.shape
        width = num_topics // 2
        unknown_topic = np.zeros(vocab_size)
        unknown_topic[width:2*width] = 1  # so topic #1 (0-indexing)
        unknown_topic /= sum(unknown_topic)
        self.assertEqual(identify_topic(unknown_topic, topics), 1)

    def test_order_topics(self):
        topics = self.topics
        unordered = topics.copy()
        self.assertTrue((topics == unordered).all())
        np.random.shuffle(unordered)
        self.assertFalse((topics == unordered).all())
        ordered = order_topics(unordered, topics)
        self.assertTrue((ordered == topics).all())

    def test_sparsify_topics(self):
        topics = self.topics
        np.testing.assert_allclose(topics, sparsify_topics(topics))

        unsparse = topics + 1e-7
        sparse = sparsify_topics(unsparse)
        np.testing.assert_allclose(topics, sparse)

    def test_dtm_conversion(self):
        topics = self.topics
        dtm = self.dtm
        num_topics, vocab_size = topics.shape
        WS, DS = horizont.utils.matrix_to_lists(dtm)
        self.assertEqual(max(WS) + 1, vocab_size)
        self.assertEqual(max(DS) + 1, NUM_DOCS)
        dtm_recovered = horizont.utils.lists_to_matrix(WS, DS)
        np.testing.assert_allclose(dtm, dtm_recovered)

    def test_LDA_random_seed(self):
        """
        Make sure we get to the same place starting from the same random seed.
        """
        dtm = self.dtm
        n_iter = 2
        random_state = 5
        fits = []
        for _ in range(2):
            clf = horizont.LDA(n_topics=NUM_TOPICS, n_iter=n_iter, random_state=random_state)
            clf.fit(dtm)
            fits.append(clf)
        np.testing.assert_array_equal(fits[0].nzw_, fits[1].nzw_)
        np.testing.assert_array_equal(fits[0].ndz_, fits[1].ndz_)
        np.testing.assert_array_equal(fits[0].theta_, fits[1].theta_)
        np.testing.assert_array_equal(fits[0].components_, fits[1].components_)
        np.testing.assert_array_equal(fits[0].loglikelihood(), fits[1].loglikelihood())

    def test_LDA(self):
        dtm = self.dtm
        n_words = np.sum(dtm)
        n_iter = 50
        # sometimes the sampler gets stuck so we adopt the following procedure
        # run the sampler with 4 random initializations and check the following:
        # 1. all samplers have log likelihood above some (lower) threshold
        # 2. at least one sampler has a log likelihood above a higher threshold
        lls = []
        for seed in range(4):
            clf = horizont.LDA(n_topics=NUM_TOPICS, n_iter=n_iter, random_state=seed)
            clf.fit(dtm)
            ll = clf.loglikelihood()
            lls.append(ll)
        for ll in lls:
            # LDA after 20 iterations should be -266000
            self.assertGreater(ll / n_words, -267000 / 1e5)
        # LDA after 100 iterations should be around -255000
        self.assertGreater(max(lls) / n_words, -255000 / 1e5)

    def test_LDA_loglikelihood(self):
        """
        Test loglikelihood calculations
        """
        dtm = self.dtm
        n_topics_true, vocab_size = self.topics.shape
        n_iter = 10
        random_state = 5
        clf = horizont.LDA(n_topics=n_topics_true, n_iter=n_iter, random_state=random_state)
        clf.fit(dtm)
        theta, phi = clf.theta_, clf.phi_
        ll = clf.loglikelihood()
        ll_cond = _loglikelihood_conditional(dtm, theta, phi)
        self.assertGreater(-240000, ll_cond)
        self.assertGreater(ll_cond, ll)

    def test_LDA_held_out_basic(self):
        """
        Test basic properties of left-to-right sequential sampler evaluation method
        """
        # simple test for consistency and monotonicity
        hold_prop = 0.05
        n_iter = 10
        R = 5
        n_topics_true, vocab_size = self.topics.shape
        dtm = self.dtm
        num_test = int(hold_prop * len(dtm))
        num_train = len(dtm) - num_test
        random_state = np.random.RandomState(5)
        dtm_train = dtm[:num_train]
        dtm_test = dtm[num_train:]

        # fit with lower n_iter
        fit = horizont.LDA(n_topics=n_topics_true, n_iter=n_iter, random_state=random_state).fit(dtm_train)

        # quick test for consistency
        logprob1 = np.sum(fit.score(dtm_test[:10], R=R, random_state=5))
        logprob2 = np.sum(fit.score(dtm_test[:10], R=R, random_state=5))
        self.assertEqual(logprob1, logprob2)

        # score lower n_iter
        logprob_orig = np.sum(fit.score(dtm_test, R=R, random_state=5))

        # test with higher n_iter
        n_iter = 20
        fit = horizont.LDA(n_topics=n_topics_true, n_iter=n_iter, random_state=random_state).fit(dtm_train)
        logprob = np.sum(fit.score(dtm_test, R=R))
        self.assertGreater(logprob, logprob_orig)



from __future__ import absolute_import, division, print_function, unicode_literals
import os
import unittest

import numpy as np

import horizont


class TestLDANews(unittest.TestCase):

    @classmethod
    def setUpClass(cls):
        test_dir = os.path.dirname(__file__)
        ap_ldac_fn = os.path.join(test_dir, 'ap.dat')
        cls.dtm = dtm = horizont.utils.ldac2dtm(open(ap_ldac_fn), offset=0)
        cls.model = model = horizont.LDA(n_topics=10, n_iter=2, random_state=1)
        cls.doctopic = model.fit_transform(dtm)

    def test_lda_news(self):
        dtm = self.dtm
        doctopic = self.doctopic
        self.assertEqual(len(doctopic), len(dtm))

    def test_lda_attributes(self):
        dtm = self.dtm
        doctopic = self.doctopic
        model = self.model

        # check dims
        N = dtm.sum()
        D, V = dtm.shape
        _, K = doctopic.shape
        self.assertEqual(model.theta_.shape, doctopic.shape)
        np.testing.assert_array_equal(model.theta_, doctopic)
        self.assertEqual(model.theta_.shape, (D, K))
        self.assertEqual(model.ndz_.shape, (D, K))
        self.assertEqual(model.phi_.shape, (K, V))
        self.assertEqual(model.nzw_.shape, (K, V))

        # check contents
        self.assertAlmostEqual(model.nzw_.sum(), N)
        self.assertAlmostEqual(model.ndz_.sum(), N)
        self.assertAlmostEqual(model.nz_.sum(), N)
        self.assertAlmostEqual(model.theta_.sum(), D)
        self.assertAlmostEqual(model.phi_.sum(), K)
        np.testing.assert_array_equal(model.ndz_.sum(axis=0), model.nz_)

        # check distributions sum to one
        np.testing.assert_array_almost_equal(model.theta_.sum(axis=1), np.ones(D))
        np.testing.assert_array_almost_equal(model.phi_.sum(axis=1), np.ones(K))



"""
The :mod:`horizont.metrics` submodule provides functions that calculate
pairwise distances.
"""
from __future__ import absolute_import, division, print_function, unicode_literals

import itertools

import numpy as np


def kl_div(X):
    """
    Considering the rows of X as discrete probability distributions, calculate
    the Kullback-Leibler divergence between each pair of vectors.

    The Kullback-Leibler (KL) divergence between two discrete probability
    distributions, ``p`` and ``q`` is given by

        :math:`D_KL(p||q) = \sum_i ln(p[i]/q[i])*p[i]

    Parameters
    ----------
    X : array, shape = [n, p]

    Returns
    -------
    distances : array, shape = [n, n]

    Examples
    --------
    >>> from horizont.metrics import kl_div
    >>> X = [[0.7, 0.3], [0.5, 0.5]]
    >>> # distance between rows of X
    >>> kl_div(X)
    array([[ 0.,  0.082282878505051782],
           [ 0.087176693572388914,  0.]])
    """

    # FIXME: look into scipy.spatial.distance.pdist
    X = np.asarray(X, dtype=float)
    n = len(X)
    distances = np.empty((n, n))
    for i, j in itertools.product(range(n), range(n)):
        distances[i, j] = 0 if i == j else np.sum(X[i]*np.log(X[i]/X[j]))
    return distances


def js_div(X):
    """Calculate Jensen–Shannon divergence. See `kl_div` for details."""
    return 0.5*(kl_div(X)+kl_div(X).T)



#-----------------------------------------------------------------------------
# Copyright (c) 2013-2014, Allen B. Riddell
#
# This file is licensed under Version 3.0 of the GNU General Public
# License. See LICENSE for a text of the license.
#-----------------------------------------------------------------------------
NAME                = 'horizont'
DESCRIPTION         = 'Dynamic topic models'
LONG_DESCRIPTION    =  open('README.rst').read()
MAINTAINER          = 'Allen B. Riddell'
MAINTAINER_EMAIL    = 'abr@ariddell.org'
URL                 = 'https://github.com/ariddell/horizont'
LICENSE             = 'GPLv3'
CLASSIFIERS = [
    'Programming Language :: Python',
    'Programming Language :: Python :: 2',
    'Programming Language :: Python :: 2.7',
    'Programming Language :: Python :: 3',
    'Programming Language :: Python :: 3.3',
    'Programming Language :: Python :: 3.4',
    'Development Status :: 3 - Alpha',
    'Environment :: Console',
    'Operating System :: OS Independent',
    'Intended Audience :: Developers',
    'Intended Audience :: Science/Research',
    'License :: OSI Approved :: GNU General Public License v3 (GPLv3)',
    'Topic :: Scientific/Engineering',
    'Topic :: Scientific/Engineering :: Information Analysis'
]

import ast
import codecs
import os
import sys

from setuptools import setup, find_packages  # before distutils import
from distutils.command.sdist import sdist
from distutils.extension import Extension

REQUIRES = ['numpy', 'scipy', 'scikit-learn']

PY2 = sys.version_info[0] == 2
if PY2:
    REQUIRES += ['futures']

# VersionFinder from from django-compressor
class VersionFinder(ast.NodeVisitor):
    def __init__(self):
        self.version = None

    def visit_Assign(self, node):
        if node.targets[0].id == '__version__':
            self.version = node.value.s


def read(*parts):
    filename = os.path.join(os.path.dirname(__file__), *parts)
    with codecs.open(filename, encoding='utf-8') as fp:
        return fp.read()


def find_version(*parts):
    finder = VersionFinder()
    finder.visit(ast.parse(read(*parts)))
    return finder.version

try:
    from Cython.Build import cythonize
    cython = True
except ImportError:
    cython = False


class CheckSDist(sdist):
    """Custom sdist that ensures Cython has compiled all pyx files to c."""

    def initialize_options(self):
        sdist.initialize_options(self)
        self._pyxfiles = []
        for root, dirs, files in os.walk('horizont'):
            for f in files:
                if f.endswith('.pyx'):
                    self._pyxfiles.append(os.path.join(root, f))

    def run(self):
        if 'cython' in cmdclass:
            self.run_command('cython')
        else:
            for pyxfile in self._pyxfiles:
                cfile = pyxfile[:-3] + 'c'
                msg = "C-source file '%s' not found." % (cfile) + \
                    " Run 'setup.py cython' before sdist."
                assert os.path.isfile(cfile), msg
        sdist.run(self)

cmdclass = {'sdist': CheckSDist}


###########################################################################
# Cython extensions to compile
###########################################################################

random_sources = ["horizont/RNG/GRNG.cpp",
                  "horizont/RNG/RNG.cpp",
                  "horizont/BayesLogit/Code/C/PolyaGamma.cpp",
                  "horizont/BayesLogit/Code/C/PolyaGammaAlt.cpp",
                  "horizont/BayesLogit/Code/C/PolyaGammaSP.cpp",
                  "horizont/BayesLogit/Code/C/InvertY.cpp"]

include_gsl_dir = os.environ.get('GSL_INC_DIR', "/usr/include/")
lib_gsl_dir = os.environ.get('GSL_LIB_DIR', "/usr/lib/")
random_include_dirs = ["horizont/BayesLogit/Code/C", "horizont/RNG", include_gsl_dir]
random_library_dirs = [lib_gsl_dir]
random_libraries = ['gsl', 'gslcblas']

# FIXME: this could be simplified, c.f. pandas
# The build will not fail if GSL cannot be found, but extensions requiring GSL
# will not work.
if cython:
    extensions = [Extension("horizont._lda", ["horizont/_lda.pyx"]),
                  Extension("horizont._random",
                            ["horizont/_random.pyx"] + random_sources,
                            include_dirs=random_include_dirs,
                            library_dirs=random_library_dirs,
                            libraries=random_libraries,
                            optional=True),
                  Extension("horizont._utils", ["horizont/_utils.pyx"])]
    extensions = cythonize(extensions)
else:
    extensions = [Extension("horizont._lda", ["horizont/_lda.c"]),
                  Extension("horizont._random",
                            ["horizont/_random.cpp"] + random_sources,
                            include_dirs=random_include_dirs,
                            library_dirs=random_library_dirs,
                            libraries=random_libraries,
                            optional=True),
                  Extension("horizont._utils", ["horizont/_utils.c"])]

import numpy
include_dirs = [numpy.get_include()]

# package data
package_data_pats = ['*.hpp', '*.pxd', '*.pyx', 'tests/*.dat', 'tests/*.ldac']

# get every file under horizont/BayesLogit/Code/C/", "horizont/RNG/"
package_data_pats += sum(
    [[os.path.join(path.replace('horizont/', ''), fn) for fn in files]
     for path, dirs, files in os.walk('horizont/BayesLogit/Code/C/')], [])

package_data_pats += sum(
    [[os.path.join(path.replace('horizont/', ''), fn) for fn in files]
     for path, dirs, files in os.walk('horizont/RNG/')], [])

###########################################################################
# Setup proper
###########################################################################

setup(install_requires=REQUIRES,
      name=NAME,
      version=find_version("horizont", "__init__.py"),
      maintainer=MAINTAINER,
      maintainer_email=MAINTAINER_EMAIL,
      packages=find_packages(),
      description=DESCRIPTION,
      long_description=LONG_DESCRIPTION,
      license=LICENSE,
      url=URL,
      classifiers=CLASSIFIERS,
      ext_modules=extensions,
      include_dirs=include_dirs,
      package_data={'horizont' : package_data_pats},
      platforms='any')



from __future__ import absolute_import, division, print_function, unicode_literals

import logging

import numpy as np
import scipy.sparse

logger = logging.getLogger('horizont')


def matrix_to_lists(doc_word):
    """Convert a (sparse) matrix of counts into arrays of word and doc indices

    Parameters
    ----------
    doc_word : array (D, V)
        document-term matrix of counts

    Returns
    -------
    (WS, DS) : tuple of two arrays
        WS[k] contains the kth word in the corpus
        DS[k] contains the document index for the kth word

    """
    if np.count_nonzero(doc_word.sum(axis=1)) != doc_word.shape[0]:
        logger.warning("all zero row in document-term matrix found")
    if np.count_nonzero(doc_word.sum(axis=0)) != doc_word.shape[1]:
        logger.warning("all zero column in document-term matrix found")
    try:
        doc_word = doc_word.tocoo()
    except AttributeError:
        doc_word = scipy.sparse.coo_matrix(doc_word)
    ii, jj, ss = doc_word.row, doc_word.col, doc_word.data
    n_tokens = int(doc_word.sum())
    DS = np.zeros(n_tokens, dtype=np.intc)
    WS = np.zeros(n_tokens, dtype=np.intc)
    startidx = 0
    for i, cnt in enumerate(ss):
        cnt = int(cnt)
        DS[startidx:startidx + cnt] = ii[i]
        WS[startidx:startidx + cnt] = jj[i]
        startidx += cnt
    return WS, DS


def lists_to_matrix(WS, DS):
    """Convert array of word (or topic) and document indices to doc-term array

    Parameters
    -----------
    (WS, DS) : tuple of two arrays
        WS[k] contains the kth word in the corpus
        DS[k] contains the document index for the kth word

    Returns
    -------
    doc_word : array (D, V)
        document-term array of counts

    """
    D = max(DS) + 1
    V = max(WS) + 1
    doc_word = np.zeros((D, V), dtype=np.intc)
    for d in range(D):
        for v in range(V):
            doc_word[d, v] = np.count_nonzero(WS[DS == d] == v)
    return doc_word


def dtm2ldac(dtm, offset=0):
    """Convert a document-term matrix into an LDA-C formatted file

    Parameters
    ----------
    dtm : array of shape N,V

    Returns
    -------
    doclines : iterable of LDA-C lines suitable for writing to file

    Note
    ----
    If a format similar to SVMLight is desired, `offset` of 1 may be used.
    """
    try:
        dtm = dtm.tocsr()
    except AttributeError:
        pass
    n_rows = dtm.shape[0]
    for i, row in enumerate(dtm):
        try:
            row = row.toarray().squeeze()
        except AttributeError:
            pass
        unique_terms = np.count_nonzero(row)
        if unique_terms == 0:
            raise ValueError("dtm contains row with all zero entries.")
        term_cnt_pairs = [(i + offset, cnt) for i, cnt in enumerate(row) if cnt > 0]
        docline = str(unique_terms) + ' '
        docline += ' '.join(["{}:{}".format(i, cnt) for i, cnt in term_cnt_pairs])
        if (i + 1) % 1000 == 0:
            logger.info("dtm2ldac: on row {} of {}".format(i + 1, n_rows))
        yield docline


def ldac2dtm(stream, offset=0):
    """Convert an LDA-C formatted file to a document-term array

    Parameters
    ----------
    stream: file object
        File yielding unicode strings in LDA-C format.

    Returns
    -------
    dtm : array of shape N,V

    Note
    ----
    If a format similar to SVMLight is the source, an `offset` of 1 may be used.
    """
    doclines = stream

    # We need to figure out the dimensions of the dtm.
    N = 0
    V = -1
    data = []
    for l in doclines:
        l = l.strip()
        # skip empty lines
        if not l:
            continue
        unique_terms = int(l.split(' ')[0])
        term_cnt_pairs = [s.split(':') for s in l.split(' ')[1:]]
        for v, _ in term_cnt_pairs:
            # check that format is indeed LDA-C with the appropriate offset
            if int(v) == 0 and offset == 1:
                raise ValueError("Indexes in LDA-C are offset 1")
        term_cnt_pairs = tuple((int(v) - offset, int(cnt)) for v, cnt in term_cnt_pairs)
        np.testing.assert_equal(unique_terms, len(term_cnt_pairs))
        V = max(V, *[v for v, cnt in term_cnt_pairs])
        data.append(term_cnt_pairs)
        N += 1
    V = V + 1
    dtm = np.zeros((N, V), dtype=np.intc)
    for i, doc in enumerate(data):
        for v, cnt in doc:
            np.testing.assert_(dtm[i, v] == 0)
            dtm[i, v] = cnt
    return dtm
